# -*- coding: UTF-8 -*-
# FlameBot, Python IRC bot modular and extensible
#           Bugzilla/Issuezilla access support
#
# Copyright (c) 2005 Diego Pettenò
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# - Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the following
#   disclaimer in the documentation and/or other materials provided
#   with the distribution.
# - The name of the author may not be used to endorse or promote
#   products derived from this software without specific prior
#   written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from xml.dom import minidom
from string import Template

# Local imports
from flamebot_utils import irc_answer, getTextForChild, FlameBotOpener
from xml.parsers.expat import ExpatError

instance = None
answer = Template("Bug ${number}; \"${short_desc}\"; ${product} | ${component}; ${bug_status}, ${resolution}; ${reporter} -> ${assigned_to}; ${url}")

class ShortBugHandler:
	def __init__(self, name, bugzilla):
		self._name = name
		self._bugzilla = bugzilla
	
	def info(self, connection, nick, channel, arg):
		self._bugzilla.infobug(connection, nick, channel, self._name + " " + arg)

class BugzillaReader:
	_bugzillas = { }
	
	def __init__(self, bot):
		bugz = minidom.parse("bugzillas.xml") # Get data about bugzillas
		for elem in (bugz.getElementsByTagName("bugzillas")[0]).getElementsByTagName("bugzilla"):
			name = str(elem.getAttribute("name"))
			self._bugzillas[name] = {}
			self._bugzillas[name]["dataurl"] = Template(str(getTextForChild(elem, "dataurl")))
			self._bugzillas[name]["reporturl"] = Template(str(getTextForChild(elem, "reporturl")))
			self._bugzillas[name]["useragent"] = str(getTextForChild(elem, "useragent"))
			
			self._bugzillas[name]["issuezilla"] = str(elem.getAttribute("issuezilla"))
			
			bot.addcommand("b_" + name, ShortBugHandler(name, self).info)
	
	def infobug(self, connection, nick, channel, args_str):
		args = args_str.split()
		
		if not self._bugzillas.has_key(args[0]):
			connection.notice(nick, "Unable to find given bugzilla")
			return 0
		if not args[1].isdigit():
			connection.notice(nick, "You have passed a non-bug#")
			return 0
		
		bugzilla = self._bugzillas[args[0]]
		
		bug_url = bugzilla["dataurl"].substitute(BUGID=args[1])
		
		print "Going to request " + bug_url
		
		bug_descriptor = FlameBotOpener(bugzilla["useragent"]).open(bug_url)
		bug_data = { 'number': args[1], 'url': bugzilla["reporturl"].substitute(BUGID=args[1]) }
		
		try:
			bug_domdoc = minidom.parse(bug_descriptor)
		except ExpatError, e:
			print "Damn, the XML is corrupt!"
			print FlameBotOpener(bugzilla["useragent"]).open(bug_url).read()
			return
		
		try:
			if bugzilla["issuezilla"] != "true":
				bug_elem = bug_domdoc.getElementsByTagName("bugzilla")[0].getElementsByTagName("bug")[0]
			else:
				bug_elem = bug_domdoc.getElementsByTagName("issuezilla")[0].getElementsByTagName("issue")[0]
		except IndexError, e:
			print "Whoa! Seems like it's protected! Dumping received data:"
			print FlameBotOpener(bugzilla["useragent"]).open(bug_url).read()
			raise
		
		for field in ['short_desc', 'product', 'component', 'bug_status', 'resolution', 'reporter', 'assigned_to']:
			bug_data[field] = getTextForChild(bug_elem, field)
		
		if bugzilla["issuezilla"] == "true":
			bug_data["bug_status"] = getTextForChild(bug_elem, "issue_status")
		
		irc_answer(connection, nick, channel, answer.substitute(bug_data))
	
	def unregister(self, bot):
		for bugzilla in self._bugzillas.keys():
			bot.removecommand("b_" + bugzilla)

def register_flamebot(bot):
	global instance
	instance = BugzillaReader(bot)
	bot.addcommand("bug", instance.infobug)

def unregister(bot):
	bot.removecommand("bug")
	instance.unregister(bot)
