# -*- coding: UTF-8 -*-
# FlameBot, Python IRC bot modular and extensible
#           Freshmeat data access
#
# Copyright (c) 2005 Diego Pettenò
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# - Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the following
#   disclaimer in the documentation and/or other materials provided
#   with the distribution.
# - The name of the author may not be used to endorse or promote
#   products derived from this software without specific prior
#   written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from xml.dom import minidom
from string import Template

# Local imports
from flamebot_utils import irc_answer, getTextForChild, FlameBotOpener
from xml.parsers.expat import ExpatError

answer = Template("${projectname_full}: ${desc_short} [Updated ${date_updated}, Vit. ${vitality_score}, Rat. ${rating}] Latest Version: ${version} . ${url_project_page}")

def cmd_freshmeat(connection, nick, channel, arg):
	fm_url = "http://freshmeat.net/projects-xml/" + arg + "/" + arg + ".xml"
	print "Going to request " + fm_url
	
	fm_descriptor = FlameBotOpener().open(fm_url)

	try:
		fm_domdoc = minidom.parse(fm_descriptor)
	except ExpatError, e:
		print "Damn, the XML is corrupt!"
		print FlameBotOpener().open(fm_url).read()
		return
	
	fm_data = { }

	try:
		fm_elem = fm_domdoc.getElementsByTagName("project-listing")[0].getElementsByTagName("project")[0]
	except IndexError, e:
		print "Whoa! Seems like it's protected! Dumping received data:"
		print FlameBotOpener().open(fm_url).read()
		return

	try:
		fm_data['version'] = getTextForChild(fm_elem.getElementsByTagName("latest_release")[0], "latest_release_version")
	except:
		print "Freshmeat has no version info..."
		fm_data['version'] = ""
		raise
	
	for field in ['projectname_full', 'desc_short', 'date_updated', 'vitality_score', 'rating', 'url_project_page']:
		fm_data[field] = getTextForChild(fm_elem, field)
	
	irc_answer(connection, nick, channel, answer.substitute(fm_data))

def register_flamebot(bot):
	bot.addcommand("freshmeat", cmd_freshmeat)

def unregister(bot):
	bot.removecommand("freshmeat")
